#! /usr/bin/env python
#
# Software License Agreement (BSD License)
#
# Copyright (c) 2012, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials provided
#    with the distribution.
#  * Neither the name of the Willow Garage nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# -*- coding: utf-8 -*-

"""
`diagnostic_updater` contains assorted Python classes to assist in diagnostic
publication. These libraries are commonly used by device drivers as part of the diagnostics
toolchain. The main parts of `diagnostic_updater` are:

 - :class:`DiagnosticStatusWrapper`, a wrapper providing
   convenience functions for working with :class:`diagnostics_msgs.msg.DiagnosticStatus`.

 - :class:`Updater`, a class for managing periodic publishing of the
   :class:`DiagnosticStatusWrapper` output by a set of :class:`DiagnosticTask`.

 - :class:`TopicDiagnostic` and :class:`HeaderlessTopicDiagnostic` for calculating and
   publishing statistics on timestamps and publication frequencies, and
   their corresponding :class:`DiagnosedPublisher` and :class:`HeaderlessDiagnosedPublisher`
   to update the statistics automatically when publications are made to a topic.

Example uses of these classes can be found in `src/example.py`.
"""

from ._diagnostic_status_wrapper import *
from ._diagnostic_updater import *
from ._update_functions import *
from ._publisher import *

__all__ = [
  'DiagnosticStatusWrapper',
  'DiagnosticTask',
  'FunctionDiagnosticTask',
  'CompositeDiagnosticTask',
  'DiagnosticTaskVector',
  'Updater',
  'HeaderlessTopicDiagnostic',
  'TopicDiagnostic',
  'DiagnosedPublisher',
  'FrequencyStatusParam',
  'FrequencyStatus',
  'TimeStampStatusParam',
  'TimeStampStatus',
  'Heartbeat',
]



# -*- coding: utf-8 -*-

import os, sys, time
from os.path import abspath, dirname, join

from docutils import nodes
from sphinx.ext import intersphinx

import catkin_pkg.package

catkin_dir = dirname(dirname(abspath(__file__)))
catkin_package = catkin_pkg.package.parse_package(join(catkin_dir, catkin_pkg.package.PACKAGE_MANIFEST_FILENAME))

project = catkin_package.name
copyright = time.strftime("%Y") + ', Czech Technical University in Prague'
author = ", ".join([a.name for a in catkin_package.authors])
version = catkin_package.version
release = catkin_package.version

import catkin_sphinx
html_theme_path = [catkin_sphinx.get_theme_path()]

# Use ROS theme
html_theme = 'ros-theme'

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.intersphinx', 'sphinx.ext.extlinks',
              'sphinx.ext.viewcode']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# Resolve ROS message and service dependencies
ros_distro = os.getenv('ROS_DISTRO', 'latest')
ros_api_base = 'https://docs.ros.org/en/%s/api/'
ros_api = ros_api_base % ros_distro

# Example configuration for intersphinx: refer to the Python standard library.
python_version = '{}.{}'.format(sys.version_info.major, sys.version_info.minor)
intersphinx_mapping = {
    'python': ('https://docs.python.org/{}/'.format(python_version), None),
    'genpy': (ros_api + 'genpy/html', None),
}

extlinks = {
    'diagnostic_msgs': (ros_api + 'diagnostic_msgs/html/msg/%s.html', 'diagnostic_msgs/'),
}

# Produce both class and constructor doc comments
autoclass_content = 'both'


def ros_msg_reference(app, env, node, contnode):
    text_node = next(iter(contnode.traverse(lambda n: n.tagname == '#text')))
    parts = text_node.astext().split('.')
    # diagnostic_msgs.msg.DiagnosticStatus
    if len(parts) == 3:
        pkg, obj_type, obj = parts
    # diagnostic_msgs.msg._DiagnosticStatus.DiagnosticStatus
    elif len(parts) == 4:
        pkg, obj_type, module, obj = parts
        if "_" + obj != module:
            return intersphinx.missing_reference(app, env, node, contnode)
    else:
        return intersphinx.missing_reference(app, env, node, contnode)
    if obj_type not in ("msg", "srv"):
        return intersphinx.missing_reference(app, env, node, contnode)
    target = ros_api + '{}/html/{}/{}.html'.format(pkg, obj_type, obj)
    ref_node = nodes.reference()
    ref_node['refuri'] = target
    title = '{}/{}'.format(pkg, obj)
    text_node = nodes.literal(title, title)
    text_node['classes'] = ['xref', 'ros', 'ros-' + obj_type]
    ref_node += text_node
    return ref_node


# Backport of fix for issue https://github.com/sphinx-doc/sphinx/issues/2549. Without it, :ivar: fields wrongly resolve cross-references.
from sphinx.domains.python import PyObject
PyObject.doc_field_types[list(map(lambda f: f.name == 'variable', PyObject.doc_field_types)).index(True)].rolename = None


def setup(app):
    app.connect("missing-reference", ros_msg_reference)

